/*======================================================================================

	Program:		Range Imaging Toolkit (RITK)
	Module:			ritkShaders.h
	Version:		$Revision: 1791 $
	Date:			$Date: 2011-09-28 11:43:43 +0200 (Wed, 28 Sep 2011) $

	Copyright (c) 2010-2011
	Pattern Recognition Lab, 
    University Erlangen-Nuremberg, Germany
    All rights reserved.

	This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See RITKCopyright.txt for more information.

	http://www5.informatik.uni-erlangen.de/research/software/ritk

========================================================================================*/

#ifndef SHADERS_H__
#define SHADERS_H___

namespace ritk
{

/**	@ingroup	Visualization
 *	@author		Jakob Wasza
 *	@brief		Fragement shader for RImages
 *
 *	@details
 *	This fragment shader is supposed to be used with the OpenGLRImageEntity class.
 *	Its purposes are to \n
 *		-	blend RGB texture and range information via a LUT \n
 *		-	per-pixel based high-peformance lighting \n
 *	Note, that instead of the range information any scalar-valued image can be used. \n
 *	Note, that this shader is supposed to be used with the OpenGLRImageEntity_VertexShader
 */
const char *OpenGLRImageEntity_FragementShader = "																															\
	uniform int m_DiscardInvalidFragments;																																	\
																																											\
	uniform int m_PrimaryPayloadClampEnabled;																																\
	uniform vec3 m_PrimaryPayloadClampMin;																																	\
	uniform vec3 m_PrimaryPayloadClampMax;																																	\
	uniform int m_PrimaryPayloadUseLUT;																																		\
	uniform sampler2D m_PrimaryPayloadTexture;																																\
	uniform sampler1D m_PrimaryPayloadLUTTexture;																															\
																																											\
	uniform int m_SecondaryPayloadClampEnabled;																																\
	uniform vec3 m_SecondaryPayloadClampMin;																																\
	uniform vec3 m_SecondaryPayloadClampMax;																																\
	uniform int m_SecondaryPayloadUseLUT;																																	\
	uniform sampler2D m_SecondaryPayloadTexture;																															\
	uniform sampler1D m_SecondaryPayloadLUTTexture;																															\
																																											\
	uniform float m_Alpha;																																					\
																																											\
	varying float ValidFlag;																																				\
	varying vec3 Normal;																																					\
	varying vec3 VertexCoord;																																				\
	varying vec3 LightVec;																																					\
																																											\
	void main()																																								\
	{																																										\
		if ( m_DiscardInvalidFragments && abs(ValidFlag-1) > 1e-6 )																											\
		{																																									\
			vec4 c;																																							\
			c.a = 0.2f;																																						\
			c.r = 128.f;																																					\
			gl_FragColor = c;																																				\
			discard;																																						\
		}						 																																			\
		else																																								\
		{																																									\
			vec4 PrimaryColor = texture2D(m_PrimaryPayloadTexture, gl_TexCoord[0].st);																						\
			if ( m_PrimaryPayloadClampEnabled )																																\
			{																																								\
				for ( int dim = 0; dim < 3; dim++ )																															\
				{																																							\
					if ( PrimaryColor[dim] < m_PrimaryPayloadClampMin[dim] )																								\
					{																																						\
						PrimaryColor[dim] = m_PrimaryPayloadClampMin[dim];																									\
					}																																						\
					if ( PrimaryColor[dim] > m_PrimaryPayloadClampMax[dim] )																								\
					{																																						\
						PrimaryColor[dim] = m_PrimaryPayloadClampMax[dim];																									\
					}																																						\
					PrimaryColor[dim] = (PrimaryColor[dim] - m_PrimaryPayloadClampMin[dim])/(m_PrimaryPayloadClampMax[dim]-m_PrimaryPayloadClampMin[dim]);					\
					PrimaryColor[dim] = PrimaryColor[dim]*(255.0/256.0);																									\
				}																																							\
			}																																								\
			if ( m_PrimaryPayloadUseLUT )																																	\
			{																																								\
				PrimaryColor = texture1D(m_PrimaryPayloadLUTTexture, (PrimaryColor.r + PrimaryColor.g + PrimaryColor.b)/3.0);												\
			}																																								\
																																											\
			vec4 SecondaryColor = texture2D(m_SecondaryPayloadTexture, gl_TexCoord[0].st);																					\
			if ( m_SecondaryPayloadClampEnabled )																															\
			{																																								\
				for ( int dim = 0; dim < 3; dim++ )																															\
				{																																							\
					if ( SecondaryColor[dim] < m_SecondaryPayloadClampMin[dim] )																							\
					{																																						\
						SecondaryColor[dim] = m_SecondaryPayloadClampMin[dim];																								\
					}																																						\
					if ( SecondaryColor[dim] > m_SecondaryPayloadClampMax[dim] )																							\
					{																																						\
						SecondaryColor[dim] = m_SecondaryPayloadClampMax[dim];																								\
					}																																						\
					SecondaryColor[dim] = (SecondaryColor[dim] - m_SecondaryPayloadClampMin[dim])/(m_SecondaryPayloadClampMax[dim]-m_SecondaryPayloadClampMin[dim]);		\
				}																																							\
			}																																								\
			if ( m_SecondaryPayloadUseLUT )																																	\
			{																																								\
				SecondaryColor = texture1D(m_SecondaryPayloadLUTTexture, (SecondaryColor.r + SecondaryColor.g + SecondaryColor.b)/3.0);										\
			}																																								\
																																											\
			vec4 RawColor = (1.0-m_Alpha)*PrimaryColor + m_Alpha*SecondaryColor;																							\
																																											\
			vec3 Eye = normalize(-VertexCoord);																																\
			vec3 Reflected = normalize(reflect(-LightVec, Normal));																											\
			vec4 IAmbient = gl_LightSource[0].ambient;																														\
			vec4 IDiffuse = gl_LightSource[0].diffuse * max(dot(Normal,LightVec), 0.0);																						\
			vec4 ISpecular = gl_LightSource[0].specular * max(dot(Reflected, Eye), 0.0);																					\
																																											\
			gl_FragColor = 0.8f* RawColor;																																	\
			gl_FragColor += 0.1f * IDiffuse;																																\
			gl_FragColor += 0.05f * ISpecular;																																\
			gl_FragColor += 0.05f * IAmbient;																																\
		}																																									\
	}																																										\
";

//gl_FragColor = gl_Color

/**	@ingroup	Visualization
 *	@author		Jakob Wasza
 *	@brief		Vertex shader for RImages
 *
 *	@details
 *	This vertex shader is supposed to be used with the OpenGLRImageEntity class.
 *	Its main purpose is to provide surface normals to a subsequent fragment shader.
 *	Normal computation is performed by numerical differentiation on a world-coord texture.
 *	Though the obtained normals are only of approximative nature, this vertex shader provides
 *	satisfying results while complying to real-time constraints.
 */
const char *OpenGLRImageEntity_VertexShader = "											\
	uniform sampler2D m_FlagTexture;													\
	uniform sampler2D m_WorldCoordTexture;												\
																						\
	varying float ValidFlag;															\
	varying vec3 Normal;																\
	varying vec3 VertexCoord;															\
	varying vec3 LightVec;																\
																						\
	void main()																			\
	{																					\
		gl_TexCoord[0] = gl_MultiTexCoord0;												\
		gl_TexCoord[1] = gl_MultiTexCoord1;												\
		gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;							\
																						\
		unsigned char Flag = 255*texture2D(m_FlagTexture, gl_TexCoord[0].st).r;			\
		ValidFlag = Flag&0x00000001;													\
																						\
		VertexCoord = vec3(gl_ModelViewMatrix * gl_Vertex);								\
		LightVec = normalize(gl_LightSource[0].position.xyz - VertexCoord);				\
																						\
		float delta = 1e-3;																\
		vec2 CenterCoord = gl_TexCoord[0].st;											\
		vec2 x_offset = vec2(delta,0);													\
		vec2 y_offset = vec2(0,delta);													\
																						\
		vec3 xmd = vec3(texture2D(m_WorldCoordTexture, gl_TexCoord[0].st - x_offset));	\
		vec3 xpd = vec3(texture2D(m_WorldCoordTexture, gl_TexCoord[0].st + x_offset));	\
		vec3 dx = (xpd-xmd)/(2*delta);													\
		vec3 ymd = vec3(texture2D(m_WorldCoordTexture, gl_TexCoord[0].st - y_offset));	\
		vec3 ypd = vec3(texture2D(m_WorldCoordTexture, gl_TexCoord[0].st + y_offset));	\
		vec3 dy = (ypd-ymd)/(2*delta);													\
		Normal.x = dx.y*dy.z - dx.z*dy.y;												\
		Normal.y = dx.z*dy.x - dx.x*dy.z;												\
		Normal.z = dx.x*dy.y - dx.y*dy.x;												\
		Normal = normalize(gl_NormalMatrix*Normal);										\
	}																					\
";

// gl_FrontColor = gl_Color;


} // end namespace ritk

#endif // SHADERS_H___